@@ -99,6 +99,7 @@ $VERSION = "5.5";
      "urllist",
      "use_sqlite",
      "username",
+     "version_timeout",
      "wait_list",
      "wget",
      "yaml_load_code",
@@ -123,8 +124,10 @@ sub edit {
     my($o,$str,$func,$args,$key_exists);
     $o = shift @args;
     if($can{$o}) {
-        $self->$o(args => \@args); # o conf init => sub init => sub load
-        return 1;
+        my $success = $self->$o(args => \@args); # o conf init => sub init => sub load
+        unless ($success) {
+            die "Panic: could not configure CPAN.pm for args [@args]. Giving up.";
+        }
     } else {
         CPAN->debug("o[$o]") if $CPAN::DEBUG;
         unless (exists $keys{$o}) {
@@ -501,12 +504,13 @@ sub home () {
     my $old_v = $CPAN::Config->{load_module_verbosity};
     $CPAN::Config->{load_module_verbosity} = q[none];
     if ($CPAN::META->has_usable("File::HomeDir")) {
-        $home = File::HomeDir->can('my_dot_config')
-            ? File::HomeDir->my_dot_config
-                : File::HomeDir->my_data;
-        unless (defined $home) {
-            $home = File::HomeDir->my_home
+        if ($^O eq 'darwin') {
+            $home = File::HomeDir->my_home; # my_data is ~/Library/Application Support on darwin,
+                                            # which causes issues in the toolchain.
         }
+        else {
+            $home = File::HomeDir->my_data || File::HomeDir->my_home;
+       }
     }
     unless (defined $home) {
         $home = $ENV{HOME};
@@ -519,18 +523,19 @@ sub load {
     my($self, %args) = @_;
     $CPAN::Be_Silent++ if $args{be_silent};
     my $doit;
-    $doit = delete $args{doit};
+    $doit = delete $args{doit} || 0;
+    $loading = 0 unless defined $loading;
 
     use Carp;
     require_myconfig_or_config;
     my @miss = $self->missing_config_data;
+    CPAN->debug("doit[$doit]loading[$loading]miss[@miss]") if $CPAN::DEBUG;
     return unless $doit || @miss;
     return if $loading;
-    $loading++;
+    local $loading = ($loading||0) + 1;
 
     require CPAN::FirstTime;
-    my($configpm,$fh,$redo);
-    $redo ||= "";
+    my($redo,$configpm,$fh);
     if (defined $INC{"CPAN/Config.pm"} && -w $INC{"CPAN/Config.pm"}) {
         $configpm = $INC{"CPAN/Config.pm"};
         $redo++;
@@ -556,9 +561,14 @@ sub load {
         if ($configpm) {
           $INC{$inc_key} = $configpm;
         } else {
-          my $text = qq{WARNING: CPAN.pm is unable to } .
-              qq{create a configuration file.};
-          output($text, 'confess');
+          my $myconfigpm = File::Spec->catfile(home,".cpan","CPAN","MyConfig.pm");
+          $CPAN::Frontend->mydie(<<"END");
+WARNING: CPAN.pm is unable to write a configuration file.  You need write
+access to your default perl library directories or you must be able to
+create and write to '$myconfigpm'.
+
+Aborting configuration.
+END
         }
 
     }
@@ -566,14 +576,14 @@ sub load {
     if ($redo && !$doit) {
         $CPAN::Frontend->myprint(<<END);
 Sorry, we have to rerun the configuration dialog for CPAN.pm due to
-some missing parameters...
+some missing parameters...  Will write to
+ <<$configpm>>
 
 END
         $args{args} = \@miss;
     }
-    CPAN::FirstTime::init($configpm, %args);
-    $loading--;
-    return;
+    my $initialized = CPAN::FirstTime::init($configpm, %args);
+    return $initialized;
 }
 
 
@@ -630,7 +640,7 @@ Edit key values as in the following (the "o" is a literal letter o):
   o conf inhibit_startup_message 1
 
 ]);
-    undef; #don't reprint CPAN::Config
+    1; #don't reprint CPAN::Config
 }
 
 sub cpl {
